/*
 * Proiect PM - Pian electric
 *
 * Tantuc Serghei - 333CB
 *
 * 29.05.2015
 */


#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/interrupt.h>

#define MAX_SIZE 256

#define NORMAL	0
#define RECORD	1
#define PLAY	2


struct note {
	uint8_t freq;
	uint32_t count;
};

static struct note song[MAX_SIZE];
static uint8_t frecv[9] = {41,   55,   69,   83,  97,  111,  125,  139, 0};
uint8_t state;
uint8_t rec = 0, play=0;

uint16_t offs_rec=0, offs_play=0;
uint32_t play_count=0;
uint8_t start=0;

static void init_timers()
{
	/* setez pinii de input pentru butoane si pinii de output pentru led-uri */
	DDRB = 0x00;
	DDRC = _BV(PC0);
	PORTB = 0xFF;
	PORTC = ~(_BV(PC0));
	DDRD = ~(_BV(PD0));
	DDRD = ~(_BV(PD1));
	PORTD |= (1 << PD0) | (1 << PD1);
	
	
	/* setez pinul de output pentru difuzor */
	DDRD |= _BV(PD5);
	TCCR1A |= _BV(WGM10) | _BV(COM1A0);
	TCCR1B |= _BV(WGM13) | _BV(CS12);

	/* setez timer-ul pentru a genera intreruperi de 10 ms */

	/* setez modul CTC */
	TCCR2A = _BV(WGM21);
	/* setez prescaler-ul */
	TCCR2B |= _BV(CS22) | _BV(CS21) | _BV(CS20);
	/* pragul la care se schimba semnalul - 10 ms*/
	OCR2A = 156;
	/* setez intreruperea */
	TIMSK1 |= (1 << OCIE1A);
}

/* sterge cantecul */
static void rm_song(struct note song[])
{
	uint16_t i;
	for (i = 0; i < offs_rec; i++) {
		song[i].freq = 0;
		song[i].count = 0;
	}
}


ISR(TIMER1_COMPA_vect)
{
	uint8_t plays = 0;
	uint8_t nota_curenta = 0;

	/* starea normala sau inregistrare */
	if (state == NORMAL || state == RECORD) {
		for (nota_curenta = 0; nota_curenta < 8; nota_curenta++) {
			if (!(PINB & _BV(nota_curenta))) {
				/* seteaza o nota */
				OCR1A = frecv[nota_curenta];
				/* seteaza pwm */
				TCCR1A |= _BV(COM1A0);
				plays = 1;
				break;
			}
		}
	}
	
	/* prima nota pentru inregistrare */
	if(state == RECORD && plays==1 && start==0)
		start=1;
	
	/* Daca nu a fost apasat niciun buton */
	if (plays == 0) {
		/* se deseteaza pwm */
		TCCR1A &= ~_BV(COM1A0);
		/* nota curenta este 0 */
		nota_curenta = 8;
	}
	

	/* in starea normala */
	if(state == NORMAL) {
		/* daca este apasat butonul pentru inregistrare */
		if ((PIND & (1 << PD0)) == 0) {
			/* flag */
			rec = 1;
		}
		else {
			/* dupa ce se ridica butonul */
			if(rec == 1) {
				/* schimba starea curenta */
				state = RECORD;
				/* sterge cantecul inregistrat */
				rm_song(song);
				offs_rec = 0;
				start=0;
				rec = 0;
			}
		}
		
		/* daca este apasat butonul pentru redare */
		if ((PIND & (1 << PD1)) == 0) {
			/* flag */
			play = 1;
		}
		else {
			/* dupa ce se ridica butonul */
			if(play == 1) {
				/* schimba starea curenta */
				state = PLAY;
				play = 0;
			}
		}
		
	} else if (state == RECORD) {
		/* stare de inregistrare */

			/* daca se apasa butonul de inregistrare */
			if ((PIND & (1 << PD0)) == 0) {
				/* flag */
				rec = 1;
			}
			else {
				/* se opreste inregistrarea */
				if (rec == 1) {
					state = NORMAL;
					rec = 0;
				}
			}
	
	} else if (state == PLAY) {
		/* starea de redare */
			/* daca se apasa butonul de redare */
			if ((PIND & (1 << PD1)) == 0) {
				play = 1;
			}
			else {
				/* se pauzeaza redarea */
				if (play == 1) {
					state = NORMAL;
					play = 0;
				}
			}
	}

	
	/* starea de inregistrare */
	if(state == PLAY) {
		/* aprinde butonul galben */
		PORTD |= (1 << PD4);
		
		/* daca este vreo melodie */
		if(offs_rec > 0) {
			/* daca curenta se mai repeta */
			if(play_count < song[offs_play].count) {
				nota_curenta = song[offs_play].freq;
				/* creste numarul nota_curentai curente */
				play_count++;
			} else {
				/* trece la nota urmatoare */
				offs_play++;
				nota_curenta = song[offs_play].freq;
				play_count = 0;
			}
		
			/* daca nu este o nota muta */
			if (nota_curenta<8) {
				/* seteaza nota curenta */
				OCR1A = frecv[nota_curenta];
				/* seteaza pwm */
				TCCR1A |= _BV(COM1A0);
			}
			else
				/* deseteaza pwm */
				TCCR1A &= ~_BV(COM1A0);
		}
		
		/* s-a terminat melodia */
		if(offs_play == offs_rec){
			state = NORMAL;
			play = 0;
			offs_play = 0;
			play_count = 0;
		}
	
	/* starea de inregistrare */
	} else if (state == RECORD){
		/* aprinde buton galben */
		PORTC |= (1 << PC0);
		
		/* prima nota ne-muta */
		if(start == 1) {
			/* daca este aceeasi nota */
			if(song[offs_rec].freq == nota_curenta) {
				/* marim numarul ei */
				song[offs_rec].count++;
			}
			else {
				/* salvam o alta nota */
				offs_rec++;
				song[offs_rec].freq = nota_curenta;
				song[offs_rec].count = 1;
			}
			/* daca nu mai este memorie */
			if (offs_rec == MAX_SIZE) {
				state = NORMAL;
				rec = 0;
			}
		}
	}
	/* se sting led-urile*/
	else {
		PORTC &= ~(1 << PC0);
		PORTD &= ~(1 << PD4);
	}

}


int main()
{
	sei();
	init_timers();

	state = NORMAL;
	while (1);

	return 0;
}
